@@ -87,6 +87,9 @@ char DSOLOCAL *get_apr_error(apr_pool_t *p, apr_status_t rc);
 
 char DSOLOCAL *get_env_var(request_rec *r, char *name);
 
+void DSOLOCAL internal_log_ex(request_rec *r, directory_config *dcfg, modsec_rec *msr,
+    int level, int fixup, const char *text, va_list ap);
+
 void DSOLOCAL internal_log(request_rec *r, directory_config *dcfg, modsec_rec *msr,
     int level, const char *text, va_list ap);
 
@@ -100,5 +103,6 @@ char DSOLOCAL *format_error_log_message(apr_pool_t *mp, error_message *em);
 
 const DSOLOCAL char *get_response_protocol(request_rec *r);
 
+
 #endif
 
