@@ -1,6 +1,6 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2008 Breach Security, Inc. (http://www.breach.com/)
+ * Copyright (c) 2004-2009 Breach Security, Inc. (http://www.breach.com/)
  *
  * This product is released under the terms of the General Public Licence,
  * version 2 (GPLv2). Please refer to the file LICENSE (included with this
@@ -16,6 +16,8 @@
  * directly using the email address support@breach.com.
  *
  */
+#include <util_filter.h>
+
 #include "modsecurity.h"
 #include "apache2.h"
 
@@ -182,14 +184,14 @@ apr_status_t read_request_body(modsec_rec *msr, char **error_msg) {
 
         rc = ap_get_brigade(r->input_filters, bb_in, AP_MODE_READBYTES, APR_BLOCK_READ, HUGE_STRING_LEN);
         if (rc != APR_SUCCESS) {
-            /* NOTE Apache returns -3 here when the request is too large
-             *      and APR_EGENERAL when the client disconnects.
+            /* NOTE Apache returns AP_FILTER_ERROR here when the request is
+             *      too large and APR_EGENERAL when the client disconnects.
              */
             switch(rc) {
                 case APR_TIMEUP :
                     *error_msg = apr_psprintf(msr->mp, "Error reading request body: %s", get_apr_error(msr->mp, rc));
                     return -4;
-                case -3 :
+                case AP_FILTER_ERROR :
                     *error_msg = apr_psprintf(msr->mp, "Error reading request body: HTTP Error 413 - Request entity too large. (Most likely.)");
                     return -3;
                 case APR_EGENERAL :
@@ -405,20 +407,28 @@ static apr_status_t send_of_brigade(modsec_rec *msr, ap_filter_t *f) {
 
     rc = ap_pass_brigade(f->next, msr->of_brigade);
     if (rc != APR_SUCCESS) {
-        int log_level = 1;
-
-        if (APR_STATUS_IS_ECONNRESET(rc)) {
-            /* Message "Connection reset by peer" is common and not a sign
-             * of something unusual. Hence we don't want to make a big deal
-             * about it, logging at NOTICE level. Everything else we log
-             * at ERROR level.
-             */
-             log_level = 3;
-        }
+        /* TODO: These need to move to flags in 2.6.  For now log them
+         * at level 4 so that they are not confusing users.
+         */
+        int log_level = 4;
 
         if (msr->txcfg->debuglog_level >= log_level) {
-            msr_log(msr, log_level, "Output filter: Error while forwarding response data (%d): %s",
-                rc, get_apr_error(msr->mp, rc));
+            switch(rc) {
+                case AP_NOBODY_WROTE :
+                    msr_log(msr, log_level, "Output filter: Error while forwarding response data (%d): No data", rc);
+                    break;
+                case AP_FILTER_ERROR :
+                    /* Look like this is caused by the error
+                     * already being handled, so we should ignore it
+                     *
+                    msr_log(msr, log_level, "Output filter: Error while forwarding response data (%d): Filter error", rc);
+                     */
+                    break;
+                default :
+                    msr_log(msr, log_level, "Output filter: Error while forwarding response data (%d): %s",
+                        rc, get_apr_error(msr->mp, rc));
+                    break;
+            }
         }
 
         return rc;
