@@ -29,6 +29,7 @@
 #include <apr_thread_proc.h>
 #include <apr_global_mutex.h>
 #include <apr_getopt.h>
+#include <apr_version.h>
 #if APR_HAVE_UNISTD_H
 #include <unistd.h>         /* for getpid() */
 #endif
@@ -77,10 +78,10 @@ do { \
 #define MEMALLOC_ERROR_MSG  "Memory allocation failed!"
 #define VERSION             MODSEC_VERSION
 
-#define CMDLINE_OPTS        "fh"
+#define CMDLINE_OPTS        "fvh"
 
-#define IN                  0
-#define OUT                 1
+#define TXIN            0
+#define TXOUT           1
 
 #define STATUSBUF_SIZE      256
 
@@ -102,7 +103,7 @@ const char logline_pattern[] =
     "\\ (\\d+)\\ (\\S+)"
     "\\ \"(.*)\"\\ \"(.*)\""
     "\\ (\\S+)\\ \"(.*)\""
-    "\\ (\\S+)\\ (\\d+)\\ (\\d+)"
+    "\\ /?(\\S+)\\ (\\d+)\\ (\\d+)"
     "\\ (\\S+)"
     "(.*)$";
 
@@ -150,9 +151,12 @@ int                          keep_entries = 0;
 const char                  *log_repository = NULL;
 void                        *logline_regex = NULL;
 int                          max_connections = 10;
+int                          max_worker_requests = 1000;
 apr_global_mutex_t          *gmutex = NULL;
 apr_thread_mutex_t          *mutex = NULL;
 apr_pool_t                  *pool = NULL;
+apr_pool_t                  *thread_pool = NULL;
+apr_pool_t                  *recv_pool = NULL;
 apr_array_header_t          *queue = NULL;
 const char                  *queue_path = NULL;
 /* apr_time_t                   queue_time = 0; */
@@ -175,7 +179,7 @@ int                          opt_force = 0;
 
 /* -- Code -- */
 
-static char *_log_escape(const char *input, apr_size_t input_len)
+static char *_log_escape(apr_pool_t *mp, const char *input, apr_size_t input_len)
 {
     static const char c2x_table[] = "0123456789abcdef";
     unsigned char *d = NULL;
@@ -184,7 +188,7 @@ static char *_log_escape(const char *input, apr_size_t input_len)
 
     if (input == NULL) return NULL;
 
-    ret = apr_palloc(pool, input_len * 4 + 1);
+    ret = apr_palloc(mp, input_len * 4 + 1);
     if (ret == NULL) return NULL;
     d = (unsigned char *)ret;
 
@@ -383,7 +387,11 @@ static void add_entry(const char *data, int start_worker)
     entry->line = strdup(data);
     entry->line_size = strlen(entry->line);
 
-    apr_thread_mutex_lock(mutex);
+    error_log(LOG_DEBUG, NULL, "Queue locking thread mutex.");
+    if (APR_STATUS_IS_EBUSY(apr_thread_mutex_trylock(mutex))) {
+        error_log(LOG_DEBUG, NULL, "Queue waiting on thread mutex.");
+        apr_thread_mutex_lock(mutex);
+    }
 
     /* Assign unique ID to this log entry. */
     entry->id = entry_counter++;
@@ -396,6 +404,7 @@ static void add_entry(const char *data, int start_worker)
         create_new_worker(0);
     }
 
+    error_log(LOG_DEBUG, NULL, "Queue unlocking thread mutex.");
     apr_thread_mutex_unlock(mutex);
 }
 
@@ -455,7 +464,7 @@ static int read_queue_entries(apr_file_t *fd, apr_time_t *queue_time)
  * Initialise the transaction log. This code should be
  * executed only once at startup.
  */
-static void transaction_log_init()
+static void transaction_log_init(void)
 {
     /* ENH: These big enough? */
     char new_queue_path[256];
@@ -467,10 +476,11 @@ static void transaction_log_init()
     apr_snprintf(old_queue_path, sizeof(old_queue_path), "%s.old", queue_path);
 
     /* Put a lock in place to ensure exclusivity. */
+    error_log(LOG_DEBUG, NULL, "Transaction initialization locking global mutex.");
     if (APR_STATUS_IS_EBUSY(apr_global_mutex_trylock(gmutex))) {
-        error_log(LOG_WARNING, NULL, "Transaction initialization waiting on mutex");
+        error_log(LOG_DEBUG, NULL, "Transaction initialization waiting on global mutex.");
+        apr_global_mutex_lock(gmutex);
     }
-    apr_global_mutex_lock(gmutex);
 
     error_log(LOG_DEBUG, NULL, "Transaction initialization started.");
 
@@ -517,14 +527,16 @@ static void transaction_log_init()
         | APR_APPEND | APR_XTHREAD, APR_OS_DEFAULT, pool) != APR_SUCCESS)
     {
         error_log(LOG_ERROR, NULL, "Failed to open the transaction log: %s\n", transaction_log_path);
+        error_log(LOG_DEBUG, NULL, "Transaction initialization unlocking global mutex.");
         apr_global_mutex_unlock(gmutex);
         logc_shutdown(1);
     }
 
-    /* Unlock */
-    apr_global_mutex_unlock(gmutex);
-
     error_log(LOG_DEBUG, NULL, "Transaction initialization completed.");
+
+    /* Unlock */
+    error_log(LOG_DEBUG, NULL, "Transaction initialization unlocking global mutex.");
+    apr_global_mutex_unlock(gmutex);
 }
 
 
@@ -537,7 +549,7 @@ static void transaction_log(int direction, const char *entry)
     char msg[8196] = "";
 
     apr_snprintf(msg, sizeof(msg), "%u %s: %s\n", (unsigned int)apr_time_sec(apr_time_now()),
-        (direction == IN ? "IN" : "OUT"), entry);
+        (direction == TXIN ? "IN" : "OUT"), entry);
     nbytes = strlen(msg);
     apr_file_write_full(transaction_log_fd, msg, nbytes, &nbytes_written);
 }
@@ -547,7 +559,7 @@ static void transaction_log(int direction, const char *entry)
  * Executes a checkpoint, which causes the current queue to be
  * written to a file and the transaction log to be truncated.
  */
-static void transaction_checkpoint()
+static void transaction_checkpoint(void)
 {
     /* ENH: These big enough? */
     char new_queue_path[256];
@@ -556,6 +568,7 @@ static void transaction_checkpoint()
     apr_hash_index_t *hi = NULL;
     char msg[256];
     int i;
+    apr_pool_t *cpool;
 
     apr_snprintf(new_queue_path, sizeof(new_queue_path), "%s.new", queue_path);
     apr_snprintf(old_queue_path, sizeof(old_queue_path), "%s.old", queue_path);
@@ -567,18 +580,23 @@ static void transaction_checkpoint()
     }
 
     /* Put a lock in place to ensure exclusivity. */
+    error_log(LOG_DEBUG, NULL, "Checkpoint locking global mutex.");
     if (APR_STATUS_IS_EBUSY(apr_global_mutex_trylock(gmutex))) {
-        error_log(LOG_WARNING, NULL, "Checkpoint waiting on mutex");
+        error_log(LOG_DEBUG, NULL, "Checkpoint waiting on global mutex.");
+        apr_global_mutex_lock(gmutex);
     }
-    apr_global_mutex_lock(gmutex);
 
     error_log(LOG_DEBUG, NULL, "Checkpoint started.");
 
+    apr_pool_create(&cpool, NULL);
+
     /* Dump active entries into a new queue file. */
     if (apr_file_open(&queue_fd, new_queue_path, APR_WRITE | APR_CREATE
-        | APR_EXCL | APR_TRUNCATE | APR_FILE_NOCLEANUP, APR_OS_DEFAULT, pool) != APR_SUCCESS)
+        | APR_EXCL | APR_TRUNCATE | APR_FILE_NOCLEANUP, APR_OS_DEFAULT, cpool) != APR_SUCCESS)
     {
         error_log(LOG_ERROR, NULL, "Failed to create file: %s", new_queue_path);
+        error_log(LOG_DEBUG, NULL, "Checkpoint unlocking global mutex.");
+        apr_pool_destroy(cpool);
         apr_global_mutex_unlock(gmutex);
         return;
     }
@@ -611,16 +629,19 @@ static void transaction_checkpoint()
     apr_file_close(queue_fd);
 
     /* Switch the files and truncate the transaction log file. */
-    apr_file_remove(old_queue_path, pool);
-    apr_file_rename(queue_path, old_queue_path, pool);
-    apr_file_rename(new_queue_path, queue_path, pool);
-    apr_file_remove(old_queue_path, pool);
+    apr_file_remove(old_queue_path, cpool);
+    apr_file_rename(queue_path, old_queue_path, cpool);
+    apr_file_rename(new_queue_path, queue_path, cpool);
+    apr_file_remove(old_queue_path, cpool);
     apr_file_trunc(transaction_log_fd, 0);
 
-    /* Unlock and exit. */
-    apr_global_mutex_unlock(gmutex);
-
     error_log(LOG_DEBUG, NULL, "Checkpoint completed.");
+
+    apr_pool_destroy(cpool);
+
+    /* Unlock and exit. */
+    error_log(LOG_DEBUG, NULL, "Checkpoint unlocking global mutex.");
+    apr_global_mutex_unlock(gmutex);
 }
 
 
@@ -677,7 +698,7 @@ static void parse_configuration_line(const char *line, int line_count)
 /**
  * Reads configuration from a file.
  */
-static void read_configuration()
+static void read_configuration(void)
 {
     char linebuf[4096];
     apr_status_t rc;
@@ -716,20 +737,19 @@ static void read_configuration()
 /**
  * Initialize the configuration.
  */
-static void init_configuration()
+static void init_configuration(void)
 {
+    char errstr[1024];
+    apr_status_t rc = 0;
     const char *s = NULL;
 
+    /* Other values may be based off the collector root. */
     s = apr_table_get(conf, "CollectorRoot");
     if (s != NULL) {
         collector_root = s;
     }
 
-    s = apr_table_get(conf, "CheckpointInterval");
-    if (s != NULL) {
-        checkpoint_interval = atoi(s);
-    }
-
+    /* Error Log */
     s = apr_table_get(conf, "ErrorLog");
     if (s != NULL) {
         error_log_path = file_path(s);
@@ -740,9 +760,45 @@ static void init_configuration()
         error_log_level = atoi(s);
     }
 
+    if ((rc = apr_file_open(&error_log_fd, error_log_path, APR_WRITE | APR_CREATE | APR_APPEND,
+        APR_OS_DEFAULT, pool)) != APR_SUCCESS)
+    {
+        error_log(LOG_ERROR, NULL, "Failed to open the error log %s: %s\n",
+            error_log_path, apr_strerror(rc, errstr, 1024));
+        logc_shutdown(1);
+    }
+
+    error_log(LOG_NOTICE, NULL, "Configuring ModSecurity Audit Log Collector %s.", VERSION);
+
+    /* Startup Delay */
+    s = apr_table_get(conf, "StartupDelay");
+    if (s != NULL) {
+        startup_delay = atoi(s);
+    }
+
+    if ( startup_delay > 0 ) {
+        error_log(LOG_NOTICE, NULL, "Delaying execution for %dms.", startup_delay);
+        apr_sleep(startup_delay * 1000);
+        error_log(LOG_DEBUG, NULL, "Continuing execution after %dms delay.", startup_delay);
+    }
+
+    /* Remaining Configuration */
+
+    error_log(LOG_DEBUG2, NULL, "CollectorRoot=%s", collector_root);
+    error_log(LOG_DEBUG2, NULL, "ErrorLog=%s", error_log_path);
+    error_log(LOG_DEBUG2, NULL, "ErrorLogLevel=%d", error_log_level);
+    error_log(LOG_DEBUG2, NULL, "StartupDelay=%d", startup_delay);
+
+    s = apr_table_get(conf, "CheckpointInterval");
+    if (s != NULL) {
+        checkpoint_interval = atoi(s);
+        error_log(LOG_DEBUG2, NULL, "CheckpointInterval=%d", checkpoint_interval);
+    }
+
     s = apr_table_get(conf, "QueuePath");
     if (s != NULL) {
         queue_path = file_path(s);
+        error_log(LOG_DEBUG2, NULL, "QueuePath=%s", queue_path);
     }
     else {
         error_log(LOG_ERROR, NULL, "QueuePath not defined in the configuration file.");
@@ -752,49 +808,59 @@ static void init_configuration()
     s = apr_table_get(conf, "LockFile");
     if (s != NULL) {
         lockfile = file_path(s);
+        error_log(LOG_DEBUG2, NULL, "LockFile=%s", lockfile);
     }
 
     s = apr_table_get(conf, "ServerErrorTimeout");
     if (s != NULL) {
         server_error_timeout = atoi(s);
-    }
-
-    s = apr_table_get(conf, "StartupDelay");
-    if (s != NULL) {
-        startup_delay = atoi(s);
+        error_log(LOG_DEBUG2, NULL, "ServerErrorTimeout=%d", server_error_timeout);
     }
 
     s = apr_table_get(conf, "TransactionDelay");
     if (s != NULL) {
         transaction_delay = atoi(s);
+        error_log(LOG_DEBUG2, NULL, "TransactionDelay=%d", transaction_delay);
     }
 
     s = apr_table_get(conf, "TransactionLog");
     if (s != NULL) {
         transaction_log_path = file_path(s);
+        error_log(LOG_DEBUG2, NULL, "TransactionLog=%s", transaction_log_path);
     }
 
     s = apr_table_get(conf, "MaxConnections");
     if (s != NULL) {
         int v = atoi(s);
         if (v >= 0) max_connections = v;
+        error_log(LOG_DEBUG2, NULL, "MaxConnections=%d", max_connections);
+    }
+
+    s = apr_table_get(conf, "MaxWorkerRequests");
+    if (s != NULL) {
+        int v = atoi(s);
+        if (v >= 0) max_worker_requests = v;
+        error_log(LOG_DEBUG2, NULL, "MaxWorkerRequests=%d", max_worker_requests);
     }
 
     s = apr_table_get(conf, "KeepAlive");
     if (s != NULL) {
         int v = atoi(s);
         if (v >= 0) keep_alive = v;
+        error_log(LOG_DEBUG2, NULL, "KeepAlive=%d", keep_alive);
     }
 
     s = apr_table_get(conf, "KeepAliveTimeout");
     if (s != NULL) {
         int v = atoi(s);
         if (v >= 0) keep_alive_timeout = v;
+        error_log(LOG_DEBUG2, NULL, "KeepAliveTimeout=%d", keep_alive_timeout);
     }
 
     s = apr_table_get(conf, "LogStorageDir");
     if (s != NULL) {
         log_repository = file_path(s);
+        error_log(LOG_DEBUG2, NULL, "LogStorageDir=%s", log_repository);
     }
     else {
         error_log(LOG_ERROR, NULL, "Missing mandatory parameter LogStorageDir.\n");
@@ -804,6 +870,7 @@ static void init_configuration()
     s = apr_table_get(conf, "ConsoleURI");
     if (s != NULL) {
         console_uri = s;
+        error_log(LOG_DEBUG2, NULL, "ConsoleURI=%s", console_uri);
     }
     else {
         error_log(LOG_ERROR, NULL, "Missing mandatory parameter ConsoleURI.\n");
@@ -813,6 +880,7 @@ static void init_configuration()
     s = apr_table_get(conf, "SensorUsername");
     if (s != NULL) {
         sensor_username = s;
+        error_log(LOG_DEBUG2, NULL, "SensorUsername=%s", sensor_username);
     }
     else {
         error_log(LOG_ERROR, NULL, "Missing mandatory parameter SensorUsername.\n");
@@ -822,6 +890,7 @@ static void init_configuration()
     s = apr_table_get(conf, "SensorPassword");
     if (s != NULL) {
         sensor_password = s;
+        error_log(LOG_DEBUG2, NULL, "SensorPassword=%s", sensor_password);
     }
     else {
         error_log(LOG_ERROR, NULL, "Missing mandatory parameter SensorPassword.\n");
@@ -835,13 +904,14 @@ static void init_configuration()
     else {
         keep_entries = 0;
     }
+    error_log(LOG_DEBUG2, NULL, "KeepEntries=%d", keep_entries);
 }
 
 
 /**
  * Clean-up resources before process shutdown.
  */
-static void logc_cleanup()
+static void logc_cleanup(void)
 {
     curl_global_cleanup();
 }
@@ -884,25 +954,30 @@ static void logc_shutdown(int rc)
 static int handle_signals(int signum)
 {
     switch (signum) {
-    case SIGHUP:
-        error_log(LOG_NOTICE, NULL, "Caught SIGHUP, ignored.");
-        /* ENH: reload config? */
-        return 0;
     case SIGINT:
         error_log(LOG_NOTICE, NULL, "Caught SIGINT, shutting down.");
         logc_shutdown(0);
     case SIGTERM:
         error_log(LOG_NOTICE, NULL, "Caught SIGTERM, shutting down.");
         logc_shutdown(0);
+#ifndef WIN32
+    case SIGHUP:
+        error_log(LOG_NOTICE, NULL, "Caught SIGHUP, ignored.");
+        /* ENH: reload config? */
+        return 0;
     case SIGALRM:
         error_log(LOG_DEBUG, NULL, "Caught SIGALRM, ignored.");
         return 0;
     case SIGTSTP:
         error_log(LOG_DEBUG, NULL, "Caught SIGTSTP, ignored.");
         return 0;
+#endif /* WIN32 */
     }
-
+#ifndef WIN32
     error_log(LOG_NOTICE, NULL, "Caught unexpected signal %d: %s", signum, apr_signal_description_get(signum));
+#else
+    error_log(LOG_NOTICE, NULL, "Caught unexpected signal %d", signum);
+#endif /* WIN32 */
     logc_shutdown(1);
 
     return 0; /* should never reach */
@@ -962,6 +1037,7 @@ size_t curl_writefunction(void *ptr, size_t size, size_t nmemb, void *stream)
 int curl_debugfunction(CURL *curl, curl_infotype infotype, char *data, size_t datalen, void *ourdata)
 {
     apr_size_t i, effectivelen;
+    apr_thread_t *thread = (apr_thread_t *)ourdata;
 
     if (error_log_level < LOG_DEBUG) return 0;
 
@@ -973,8 +1049,24 @@ int curl_debugfunction(CURL *curl, curl_infotype infotype, char *data, size_t da
         }
     }
 
-    if (infotype == CURLINFO_TEXT) {
-        error_log(LOG_DEBUG, ourdata, "CURL: %s", _log_escape(data, effectivelen));
+    if (error_log_level >= LOG_DEBUG) {
+        if (infotype == CURLINFO_TEXT) {
+            error_log(LOG_DEBUG, thread, "CURL: %s", _log_escape(apr_thread_pool_get(thread), data, effectivelen));
+        }
+    }
+    if (error_log_level >= LOG_DEBUG2) {
+        if (infotype == CURLINFO_HEADER_IN) {
+            error_log(LOG_DEBUG2, thread, "CURL: HEADER_IN %s", _log_escape(apr_thread_pool_get(thread), data, effectivelen));
+        }
+        else if (infotype == CURLINFO_HEADER_OUT) {
+            error_log(LOG_DEBUG2, thread, "CURL: HEADER_OUT %s", _log_escape(apr_thread_pool_get(thread), data, effectivelen));
+        }
+        else if (infotype == CURLINFO_DATA_IN) {
+            error_log(LOG_DEBUG2, thread, "CURL: DATA_IN %s", _log_escape(apr_thread_pool_get(thread), data, effectivelen));
+        }
+        else if (infotype == CURLINFO_DATA_OUT) {
+            error_log(LOG_DEBUG2, thread, "CURL: DATA_OUT %s", _log_escape(apr_thread_pool_get(thread), data, effectivelen));
+        }
     }
 
     return 0;
@@ -984,31 +1076,13 @@ int curl_debugfunction(CURL *curl, curl_infotype infotype, char *data, size_t da
 /**
  * Initialise the necessary resources and structures.
  */
-static void logc_init()
+static void logc_init(void)
 {
     char errstr[1024];
     apr_status_t rc = 0;
     const char *errptr = NULL;
     int i, erroffset;
 
-    curl_global_init(CURL_GLOBAL_ALL);
-    atexit(logc_cleanup);
-
-    if ((rc = apr_file_open(&error_log_fd, error_log_path, APR_WRITE | APR_CREATE | APR_APPEND,
-        APR_OS_DEFAULT, pool)) != APR_SUCCESS)
-    {
-        error_log(LOG_ERROR, NULL, "Failed to open the error log %s: %s\n",
-            error_log_path, apr_strerror(rc, errstr, 1024));
-        logc_shutdown(1);
-    }
-
-    if ( startup_delay > 0 ) {
-        error_log(LOG_NOTICE, NULL, "ModSecurity Audit Log Collector %s delaying startup for %dms", VERSION, startup_delay);
-        apr_sleep(startup_delay * 1000);
-    }
-
-    error_log(LOG_NOTICE, NULL, "ModSecurity Audit Log Collector %s started.", VERSION);
-
     queue = apr_array_make(pool, 64, sizeof(entry_t *));
     if (queue == NULL) {
         error_log(LOG_ERROR, NULL, MEMALLOC_ERROR_MSG);
@@ -1028,7 +1102,7 @@ static void logc_init()
     }
 
     if ((rc = apr_thread_mutex_create(&mutex, APR_THREAD_MUTEX_UNNESTED, pool)) != APR_SUCCESS) {
-        error_log(LOG_ERROR, NULL, "Failed to create mutex: %s",
+        error_log(LOG_ERROR, NULL, "Failed to create thread mutex: %s",
             apr_strerror(rc, errstr, 1024));
         logc_shutdown(1);
     }
@@ -1050,12 +1124,13 @@ static void logc_init()
 
         /* Pre-configure the handle. */
         curl_easy_setopt(curl, CURLOPT_UPLOAD, TRUE);
-        curl_easy_setopt(curl, CURLOPT_PUT, TRUE);
         curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, (char *)NULL);
         curl_easy_setopt(curl, CURLOPT_URL, console_uri);
         curl_easy_setopt(curl, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
         curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, FALSE);
         curl_easy_setopt(curl, CURLOPT_SSL_VERIFYHOST, 0);
+        /* SSLv3 works better overall as some servers have issues with TLS */
+        curl_easy_setopt(curl, CURLOPT_SSLVERSION, CURL_SSLVERSION_SSLv3);
         curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 15);
         curl_easy_setopt(curl, CURLOPT_NOSIGNAL, TRUE);
         curl_easy_setopt(curl, CURLOPT_HEADER, TRUE);
@@ -1120,6 +1195,10 @@ static void keep_entries_hack(apr_pool_t *mp, apr_thread_t *thread, const char *
         return;
     }
 
+    if (error_log_level >= LOG_DEBUG) {
+        error_log(LOG_DEBUG, thread, "STAT \"%s\" {uid=%d; gid=%d; size=%" APR_OFF_T_FMT "; csize=%" APR_OFF_T_FMT "; atime=%" APR_TIME_T_FMT "; ctime=%" APR_TIME_T_FMT "; mtime=%" APR_TIME_T_FMT "}", fn, finfo.user, finfo.group, finfo.size, finfo.csize, finfo.atime, finfo.ctime, finfo.mtime);
+    }
+
     if (finfo.mtime != KEEP_ENTRIES_REMOVE_TIME) {
         error_log(LOG_DEBUG2, thread, "Set mtime: %s", fn);
         if ((rc = apr_file_mtime_set(fn, (apr_time_t)KEEP_ENTRIES_REMOVE_TIME, mp)) != APR_SUCCESS) {
@@ -1155,6 +1234,7 @@ static void * APR_THREAD_FUNC thread_worker(apr_thread_t *thread, void *data)
     apr_pool_t *tpool;
     struct curl_slist *headerlist = NULL;
     char curl_error_buffer[CURL_ERROR_SIZE] = "";
+    int num_requests = 0;
 
     /* There is no need to do the sleep if this was an invalid entry
      * as the sleep is just to protect flooding the console server
@@ -1164,13 +1244,13 @@ static void * APR_THREAD_FUNC thread_worker(apr_thread_t *thread, void *data)
     int nodelay = 0;
 
 
+    error_log(LOG_DEBUG, thread, "Worker thread starting.");
+
     /* Each worker uses its own pool to manage memory. To avoid
      * memory leaks the pool is cleared after each processed
      * entry.
      */
-    apr_pool_create(&tpool, NULL);
-
-    error_log(LOG_DEBUG, thread, "Worker thread starting.");
+    apr_pool_create(&tpool, thread_pool);
 
     /* Process jobs in a queue until there are no more jobs to process. */
     for(;;) {
@@ -1197,14 +1277,18 @@ static void * APR_THREAD_FUNC thread_worker(apr_thread_t *thread, void *data)
 
         /* Get a new entry, but only if we need one. */
         if (take_new) {
-            error_log(LOG_DEBUG, thread, "Locking mutex.");
+            error_log(LOG_DEBUG, thread, "Worker fetch locking thread mutex.");
+            if (APR_STATUS_IS_EBUSY(apr_thread_mutex_trylock(mutex))) {
+                error_log(LOG_DEBUG, thread, "Worker fetch waiting on thread mutex.");
+                apr_thread_mutex_lock(mutex);
+            }
 
-            apr_thread_mutex_lock(mutex);
+            error_log(LOG_DEBUG, thread, "Worker fetch started.");
 
             /* Deal with the previous entry. */
             if (entry != NULL) {
                 error_log(LOG_DEBUG, thread, "Removing previous entry from storage.");
-                transaction_log(OUT, entry->line);
+                transaction_log(TXOUT, entry->line);
 
                 /* Remove previous entry from storage. */
                 apr_hash_set(in_progress, &entry->id, sizeof(entry->id), NULL);
@@ -1220,17 +1304,19 @@ static void * APR_THREAD_FUNC thread_worker(apr_thread_t *thread, void *data)
             /* Get one entry. */
             entryptr = (entry_t **)apr_array_pop(queue);
             if (entryptr == NULL) {
+                error_log(LOG_DEBUG, thread, "Worker fetch unlocking thread mutex.");
                 apr_thread_mutex_unlock(mutex);
                 error_log(LOG_DEBUG, thread, "No more work for this thread, exiting.");
 
                 goto THREAD_SHUTDOWN;
             }
-            else {
-                error_log(LOG_DEBUG, thread, "Got one job.");
-                entry = *entryptr;
-                apr_hash_set(in_progress, &entry->id, sizeof(entry->id), entry);
-            }
 
+            entry = *entryptr;
+            apr_hash_set(in_progress, &entry->id, sizeof(entry->id), entry);
+
+            error_log(LOG_DEBUG, thread, "Worker fetch completed.");
+
+            error_log(LOG_DEBUG, thread, "Worker fetch unlocking thread mutex.");
             apr_thread_mutex_unlock(mutex);
         }
 
@@ -1239,15 +1325,20 @@ static void * APR_THREAD_FUNC thread_worker(apr_thread_t *thread, void *data)
         error_log(LOG_DEBUG, thread, "Processing entry.");
         take_new = 0;
 
+        /* Keep track of requests processed if we need to */
+        if (max_worker_requests > 0) {
+            num_requests++;
+        }
+
         rc = pcre_exec(logline_regex, NULL, entry->line, entry->line_size, 0, 0,
             capturevector, CAPTUREVECTORSIZE);
         if (rc == PCRE_ERROR_NOMATCH) { /* No match. */
-            error_log(LOG_WARNING, thread, "Invalid entry (failed to match regex): %s", _log_escape(entry->line, entry->line_size));
+            error_log(LOG_WARNING, thread, "Invalid entry (failed to match regex): %s", _log_escape(tpool, entry->line, entry->line_size));
             take_new = 1;
             nodelay = 1;
         }
         else if (rc < 0) { /* Error condition. */
-            error_log(LOG_WARNING, thread, "Invalid entry (PCRE error %d): %s", rc, _log_escape(entry->line, entry->line_size));
+            error_log(LOG_WARNING, thread, "Invalid entry (PCRE error %d): %s", rc, _log_escape(tpool, entry->line, entry->line_size));
             take_new = 1;
             nodelay = 1;
         }
@@ -1278,11 +1369,24 @@ static void * APR_THREAD_FUNC thread_worker(apr_thread_t *thread, void *data)
                 char response_buf[STATUSBUF_SIZE];
                 CURLcode res;
 
+                if (error_log_level >= LOG_DEBUG) {
+                    error_log(LOG_DEBUG, thread, "STAT \"%s\" {uid=%d; gid=%d; size=%" APR_OFF_T_FMT "; csize=%" APR_OFF_T_FMT "; atime=%" APR_TIME_T_FMT "; ctime=%" APR_TIME_T_FMT "; mtime=%" APR_TIME_T_FMT "}", auditlogentry, finfo.user, finfo.group, finfo.size, finfo.csize, finfo.atime, finfo.ctime, finfo.mtime);
+                }
+
                 /* Initialize the respone buffer with a hidden value */
                 response_buf[0] = 0;
                 response_buf[1] = 1;
 
-                error_log(LOG_DEBUG, thread, "File found, activating cURL.");
+                if (finfo.size == 0) {
+                    error_log(LOG_WARNING, thread, "File found (%" APR_SIZE_T_FMT " bytes), skipping.", finfo.size);
+                    take_new = 1;
+                    nodelay = 1;
+                    goto THREAD_CLEANUP;
+                }
+                else {
+                    error_log(LOG_DEBUG, thread, "File found (%" APR_SIZE_T_FMT " bytes), activating cURL.", finfo.size);
+                }
+
 
                 curl_easy_setopt(curl, CURLOPT_VERBOSE, 1);
                 curl_easy_setopt(curl, CURLOPT_DEBUGFUNCTION, curl_debugfunction);
@@ -1306,7 +1410,6 @@ static void * APR_THREAD_FUNC thread_worker(apr_thread_t *thread, void *data)
 
                 curl_easy_setopt(curl, CURLOPT_READDATA, hd_src);
                 curl_easy_setopt(curl, CURLOPT_INFILESIZE_LARGE, finfo.size);
-                curl_easy_setopt(curl, CURLOPT_INFILESIZE, finfo.size);
 #if 0
                 mandatory on win32?
                 curl_easy_setopt(curl, CURLOPT_READFUNCTION, read_callback);
@@ -1395,6 +1498,15 @@ static void * APR_THREAD_FUNC thread_worker(apr_thread_t *thread, void *data)
                 take_new = 1;
                 nodelay = 1;
             }
+
+            /* If we are tracking num_requests, then shutdown if we are
+             * over our threshold.
+             */
+            if (num_requests && (num_requests >= max_worker_requests)) {
+                error_log(LOG_NOTICE, thread, "Reached max requests (%d) for this worker, exiting.", max_worker_requests);
+
+                goto THREAD_SHUTDOWN;
+            }
         }
 
         THREAD_CLEANUP:
@@ -1413,12 +1525,16 @@ static void * APR_THREAD_FUNC thread_worker(apr_thread_t *thread, void *data)
 
         apr_pool_clear(tpool);
 
-        error_log(LOG_DEBUG, thread, "Loop completed.");
+        error_log(LOG_DEBUG, thread, "Worker processing completed.");
     }
 
     THREAD_SHUTDOWN:
 
-    apr_thread_mutex_lock(mutex);
+    error_log(LOG_DEBUG, thread, "Worker shutdown locking thread mutex.");
+    if (APR_STATUS_IS_EBUSY(apr_thread_mutex_trylock(mutex))) {
+        error_log(LOG_DEBUG, thread, "Worker shutdown waiting on thread mutex.");
+        apr_thread_mutex_lock(mutex);
+    }
 
     /* Deal with the previous entry, if any. */
     if (entry != NULL) {
@@ -1428,7 +1544,7 @@ static void * APR_THREAD_FUNC thread_worker(apr_thread_t *thread, void *data)
             *(entry_t **)apr_array_push(queue) = entry;
         }
         else {
-            transaction_log(OUT, entry->line);
+            transaction_log(TXOUT, entry->line);
             free((void *)entry->line);
             free(entry);
         }
@@ -1442,11 +1558,13 @@ static void * APR_THREAD_FUNC thread_worker(apr_thread_t *thread, void *data)
     /* No more work, exit. */
     current_workers--;
 
+    error_log(LOG_DEBUG, thread, "Worker shutdown unlocking thread mutex.");
     apr_thread_mutex_unlock(mutex);
 
     apr_pool_destroy(tpool);
 
-    error_log(LOG_DEBUG, thread, "Thread done.");
+    error_log(LOG_DEBUG, thread, "Worker thread completed.");
+
     apr_thread_exit(thread, 0);
 
     return NULL;
@@ -1462,42 +1580,75 @@ static void create_new_worker(int lock)
     apr_thread_t *thread = NULL;
     CURL **curlptr = NULL;
 
-    if (lock) apr_thread_mutex_lock(mutex);
+    if (lock) {
+        error_log(LOG_DEBUG, NULL, "Worker creation locking thread mutex.");
+        if (APR_STATUS_IS_EBUSY(apr_thread_mutex_trylock(mutex))) {
+            error_log(LOG_DEBUG, NULL, "Worker creation waiting on thread mutex.");
+            apr_thread_mutex_lock(mutex);
+        }
+    }
+
+    error_log(LOG_DEBUG, NULL, "Worker creation started.");
 
     /* A sanity check: this part executes under lock and
      * we want to make *sure* we don't create more threads
      * than we are allowed.
      */
     if (current_workers >= max_connections) {
-        if (lock) apr_thread_mutex_unlock(mutex);
+        if (lock) {
+            error_log(LOG_DEBUG, NULL, "Worker creation unlocking thread mutex.");
+            apr_thread_mutex_unlock(mutex);
+        }
         return;
     }
 
+    /* Cleanup thread pool when idle */
+    if (current_workers <= 0)  {
+        if (thread_pool != NULL) {
+            error_log(LOG_DEBUG, NULL, "Destroying thread_pool.");
+            apr_pool_destroy(thread_pool);
+            thread_pool = NULL;
+        }
+        error_log(LOG_DEBUG, NULL, "Creating thread_pool.");
+        apr_pool_create(&thread_pool, NULL);
+    }
+
     curlptr = (CURL **)apr_array_pop(curl_handles);
     if (curlptr != NULL) {
         apr_threadattr_t *thread_attrs;
         apr_status_t rc;
 
-        apr_threadattr_create(&thread_attrs, pool);
+        apr_threadattr_create(&thread_attrs, thread_pool);
         apr_threadattr_detach_set(thread_attrs, 1);
         apr_threadattr_stacksize_set(thread_attrs, 1024);
 
-        rc = apr_thread_create(&thread, thread_attrs, thread_worker, *curlptr, pool);
+        rc = apr_thread_create(&thread, thread_attrs, thread_worker, *curlptr, thread_pool);
         if (rc != APR_SUCCESS) {
-            apr_thread_mutex_unlock(mutex);
-            error_log(LOG_ERROR, thread, "Failed to create new worker thread: %d", rc);
+            if (lock) {
+                error_log(LOG_DEBUG, NULL, "Worker creation unlocking thread mutex.");
+                apr_thread_mutex_unlock(mutex);
+            }
+            error_log(LOG_ERROR, NULL, "Failed to create new worker thread: %d", rc);
             logc_shutdown(1);
         }
 
         current_workers++;
     }
     else {
-        if (lock) apr_thread_mutex_unlock(mutex);
-        error_log(LOG_ERROR, thread, "No more cURL handles (Internal Error).");
+        if (lock) {
+            error_log(LOG_DEBUG, NULL, "Worker creation unlocking thread mutex.");
+            apr_thread_mutex_unlock(mutex);
+        }
+        error_log(LOG_ERROR, NULL, "No more cURL handles (Internal Error).");
         logc_shutdown(1);
     }
 
-    if (lock) apr_thread_mutex_unlock(mutex);
+    error_log(LOG_DEBUG, NULL, "Worker creation completed: %pp", thread);
+
+    if (lock) {
+        error_log(LOG_DEBUG, NULL, "Worker creation unlocking thread mutex.");
+        apr_thread_mutex_unlock(mutex);
+    }
 }
 
 
@@ -1577,7 +1728,7 @@ static void * APR_THREAD_FUNC thread_manager(apr_thread_t *thread, void *data)
     return NULL;
 }
 
-
+#ifndef WIN32
 /**
  * Thread to handle all signals
  */
@@ -1594,14 +1745,14 @@ static void * APR_THREAD_FUNC thread_signals(apr_thread_t *thread, void *data)
 
     return NULL;
 }
-
+#endif /* WIN32 */
 
 /**
  * The main loop where we receive log entries from
  * Apache and add them to the queue, sometimes creating
  * new worker threads to handle them.
  */
-static void receive_loop() {
+static void receive_loop(void) {
     apr_file_t *fd_stdin;
     apr_size_t nbytes = PIPE_BUF_SIZE;
     char *buf = apr_palloc(pool, PIPE_BUF_SIZE + 1);
@@ -1612,6 +1763,8 @@ static void receive_loop() {
     int done = 0;
     int drop_next = 0;
     int buffered_events = 0;
+    int count = 0;
+    apr_pool_t *tmp_pool;
 
     /* Open stdin. */
     if (apr_file_open_stdin(&fd_stdin, pool) != APR_SUCCESS) {
@@ -1622,11 +1775,15 @@ static void receive_loop() {
     /* Always want this NUL terminated */
     buf[PIPE_BUF_SIZE] = '\0';
 
+    apr_pool_create(&tmp_pool, NULL);
+
     /* Loop forever receiving entries from stdin. */
     while(!done || (curr < next)) {
         apr_status_t rc;
 
-        error_log(LOG_DEBUG2, NULL, "Internal state: [evnt \"%" APR_SIZE_T_FMT "\"][curr \"%" APR_SIZE_T_FMT "\"][next \"%" APR_SIZE_T_FMT "\"][nbytes \"%" APR_SIZE_T_FMT "\"]", evnt, curr, next, nbytes);
+        if (error_log_level >= LOG_DEBUG2) {
+            error_log(LOG_DEBUG2, NULL, "Internal state: [evnt \"%" APR_SIZE_T_FMT "\"][curr \"%" APR_SIZE_T_FMT "\"][next \"%" APR_SIZE_T_FMT "\"][nbytes \"%" APR_SIZE_T_FMT "\"]", evnt, curr, next, nbytes);
+        }
 
         /* If we are not done and have the space, read more */
         if (!done && (nbytes > 0)) {
@@ -1645,7 +1802,7 @@ static void receive_loop() {
                     error_log(LOG_DEBUG, NULL, "Read %" APR_SIZE_T_FMT " bytes from pipe", nbytes);
                 }
                 else {
-                    error_log(LOG_DEBUG2, NULL, "Read %" APR_SIZE_T_FMT " bytes from pipe: `%s'", nbytes, _log_escape((buf + next), nbytes));
+                    error_log(LOG_DEBUG2, NULL, "Read %" APR_SIZE_T_FMT " bytes from pipe: `%s'", nbytes, _log_escape(tmp_pool, (buf + next), nbytes));
                 }
             }
 
@@ -1669,13 +1826,13 @@ static void receive_loop() {
 
                 /* We may have to drop this one if it previously failed */
                 if (drop_next) {
-                    error_log(LOG_ERROR, NULL, "Dropping remaining portion of failed event: `%s'", _log_escape((buf + evnt), (curr - evnt)));
+                    error_log(LOG_ERROR, NULL, "Dropping remaining portion of failed event: `%s'", _log_escape(tmp_pool, (buf + evnt), (curr - evnt)));
                     drop_next = 0;
                 }
                 else {
-                    transaction_log(IN, buf + evnt);
+                    transaction_log(TXIN, buf + evnt);
                     error_log(LOG_DEBUG2, NULL, "Received audit log entry (count %lu queue %d workers %d): %s",
-                        entry_counter, queue->nelts, current_workers, _log_escape((buf + evnt), strlen(buf + evnt)));
+                        entry_counter, queue->nelts, current_workers, _log_escape(tmp_pool, (buf + evnt), strlen(buf + evnt)));
                     add_entry(buf + evnt, 1);
                     buffered_events++;
                 }
@@ -1684,7 +1841,7 @@ static void receive_loop() {
                 evnt = curr = curr + 1;
             }
             else {
-                error_log(LOG_DEBUG2, NULL, "Event buffer contains partial event: `%s'", _log_escape((buf + evnt), (next - evnt)));
+                error_log(LOG_DEBUG2, NULL, "Event buffer contains partial event: `%s'", _log_escape(tmp_pool, (buf + evnt), (next - evnt)));
                 break;
             }
         }
@@ -1698,7 +1855,7 @@ static void receive_loop() {
             curr -= evnt;
             memmove(buf, (buf + evnt), next);
 
-            error_log(LOG_DEBUG2, NULL, "Shifted buffer back %" APR_SIZE_T_FMT " and offset %" APR_SIZE_T_FMT " bytes for next read: `%s'", evnt, next, _log_escape(buf, next));
+            error_log(LOG_DEBUG2, NULL, "Shifted buffer back %" APR_SIZE_T_FMT " and offset %" APR_SIZE_T_FMT " bytes for next read: `%s'", evnt, next, _log_escape(tmp_pool, buf, next));
 
             evnt = 0;
         }
@@ -1710,10 +1867,10 @@ static void receive_loop() {
              */
 
             if (drop_next) {
-                error_log(LOG_ERROR, NULL, "Event continuation too large, dropping it as well: `%s'", _log_escape(buf, PIPE_BUF_SIZE));
+                error_log(LOG_ERROR, NULL, "Event continuation too large, dropping it as well: `%s'", _log_escape(tmp_pool, buf, PIPE_BUF_SIZE));
             }
             else {
-                error_log(LOG_ERROR, NULL, "Event too large, dropping event: `%s'", _log_escape(buf, PIPE_BUF_SIZE));
+                error_log(LOG_ERROR, NULL, "Event too large, dropping event: `%s'", _log_escape(tmp_pool, buf, PIPE_BUF_SIZE));
             }
 
             /* Rewind buf and mark that we need to drop up to the next event */
@@ -1722,6 +1879,16 @@ static void receive_loop() {
         }
 
         nbytes = PIPE_BUF_SIZE - next;
+
+        if (count++ > 1000) {
+            count = 0;
+            error_log(LOG_DEBUG, NULL, "Recycling tmp_pool.");
+            apr_pool_destroy(tmp_pool);
+            apr_pool_create(&tmp_pool, NULL);
+        }
+        else {
+            apr_pool_clear(tmp_pool);
+        }
     }
 
     /* Wait for queue to empty if specified */
@@ -1740,7 +1907,7 @@ static void receive_loop() {
 /**
  * Creates the management thread.
  */
-static void start_management_thread()
+static void start_management_thread(void)
 {
     apr_thread_t *thread = NULL;
     apr_threadattr_t *thread_attrs;
@@ -1759,11 +1926,11 @@ static void start_management_thread()
         logc_shutdown(1);
     }
 }
-
+#ifndef WIN32
 /**
  * Creates a thread to handle all signals
  */
-static void start_signal_thread()
+static void start_signal_thread(void)
 {
     apr_thread_t *thread = NULL;
     apr_threadattr_t *thread_attrs;
@@ -1779,19 +1946,31 @@ static void start_signal_thread()
         logc_shutdown(1);
     }
 }
+#endif /* WIN32 */
 
 /**
  * Usage text.
  */
-static void usage() {
-    fprintf(stderr, "ModSecurity Log Collector v%s\n", VERSION);
-    fprintf(stderr, "  Usage: mlogc [options] /path/to/the/configuration.file\n");
+static void usage(void) {
+    fprintf(stderr, "ModSecurity Log Collector (mlogc) v%s\n", VERSION);
+    fprintf(stderr, "  Usage: mlogc [options] /path/to/the/mlogc.conf\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "  Options:\n");
     fprintf(stderr, "    -f        Force depletion of queue on exit\n");
+    fprintf(stderr, "    -v        Version information\n");
     fprintf(stderr, "    -h        This help\n\n");
 }
 
+/**
+ * Version text.
+ */
+static void version(void) {
+    fprintf(stderr, "ModSecurity Log Collector (mlogc) v%s\n", VERSION);
+    fprintf(stderr, "   APR: compiled=\"%s\"; loaded=\"%s\"\n", APR_VERSION_STRING, apr_version_string());
+    fprintf(stderr, "  PCRE: compiled=\"%d.%d\"; loaded=\"%s\"\n", PCRE_MAJOR, PCRE_MINOR, pcre_version());
+    fprintf(stderr, "  cURL: compiled=\"%s\"; loaded=\"%s\"\n", LIBCURL_VERSION, curl_version());
+    fprintf(stderr, "\n");
+}
 
 /**
  * This is the main entry point.
@@ -1803,9 +1982,19 @@ int main(int argc, const char * const argv[]) {
     apr_app_initialize(&argc, &argv, NULL);
     atexit(apr_terminate);
 
+    curl_global_init(CURL_GLOBAL_ALL);
+    atexit(logc_cleanup);
+
     logc_pid = getpid();
     apr_pool_create(&pool, NULL);
+    apr_pool_create(&recv_pool, NULL);
+	
+#ifndef WIN32
     apr_setup_signal_thread();
+#else
+	apr_signal(SIGINT, handle_signals);
+	apr_signal(SIGTERM, handle_signals);
+#endif /* WIN32 */
 
     if (argc < 2) {
         usage();
@@ -1829,6 +2018,9 @@ int main(int argc, const char * const argv[]) {
                 case 'f':
                     opt_force = 1;
                     break;
+                case 'v':
+                    version();
+                    logc_shutdown(0);
                 case 'h':
                     usage();
                     logc_shutdown(0);
@@ -1854,7 +2046,9 @@ int main(int argc, const char * const argv[]) {
     server_error = 0;
 
     start_management_thread();
+#ifndef WIN32
     start_signal_thread();
+#endif /* WIN32 */
 
     /* Process stdin until EOF */
     receive_loop();
