@@ -1,6 +1,6 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2008 Breach Security, Inc. (http://www.breach.com/)
+ * Copyright (c) 2004-2009 Breach Security, Inc. (http://www.breach.com/)
  *
  * This product is released under the terms of the General Public Licence,
  * version 2 (GPLv2). Please refer to the file LICENSE (included with this
@@ -25,6 +25,7 @@
 #include "apache2.h"
 #include "http_main.h"
 #include "pdf_protect.h"
+
 #include "msc_logging.h"
 #include "msc_util.h"
 
@@ -520,7 +521,7 @@ static int hook_post_config(apr_pool_t *mp, apr_pool_t *mp_log, apr_pool_t *mp_t
     /* Log our presence to the error log. */
     if (first_time) {
         ap_log_error(APLOG_MARK, APLOG_NOTICE | APLOG_NOERRNO, 0, s,
-                "%s configured.", MODULE_NAME_FULL);
+                "%s configured.", MODSEC_MODULE_NAME_FULL);
 
         /* If we've changed the server signature make note of the original. */
         if (new_server_signature != NULL) {
@@ -570,19 +571,24 @@ static int hook_request_early(request_rec *r) {
         return DECLINED;
     }
 
-    /* Check request body limit (should only trigger on non-chunked requests). */
-    if (msr->request_content_length > msr->txcfg->reqbody_limit) {
-        msr_log(msr, 1, "Request body is larger than the "
-                     "configured limit (%ld).", msr->txcfg->reqbody_limit);
-        return HTTP_REQUEST_ENTITY_TOO_LARGE;
-    }
-
     /* Process phase REQUEST_HEADERS */
     rc = DECLINED;
     if (modsecurity_process_phase(msr, PHASE_REQUEST_HEADERS) > 0) {
         rc = perform_interception(msr);
     }
 
+    if (    (msr->txcfg->is_enabled != MODSEC_DISABLED)
+         && (msr->txcfg->reqbody_access == 1)
+         && (rc == DECLINED))
+    {
+        /* Check request body limit (non-chunked requests only). */
+        if (msr->request_content_length > msr->txcfg->reqbody_limit) {
+            msr_log(msr, 1, "Request body (Content-Length) is larger than the "
+                         "configured limit (%ld).", msr->txcfg->reqbody_limit);
+            return HTTP_REQUEST_ENTITY_TOO_LARGE;
+        }
+    }
+
     return rc;
 }
 
@@ -1045,7 +1051,7 @@ static void modsec_register_variable(const char *name, unsigned int type,
                                      void *fn_validate, void *fn_generate,
                                      unsigned int is_cacheable, unsigned int availability) {
     if (modsecurity != NULL) {
-        msre_engine_variable_register(modsecurity->msre, name, type, argc_min, argc_max, fn_validate, fn_generate, is_cacheable, availability);
+        msre_engine_variable_register(modsecurity->msre, name, type, argc_min, argc_max, (fn_var_validate_t)fn_validate, (fn_var_generate_t)fn_generate, is_cacheable, availability);
     }
     else {
         fprintf(stderr,"modsecurity is NULL\n");
@@ -1120,8 +1126,18 @@ static void register_hooks(apr_pool_t *mp) {
 
     ap_register_input_filter("MODSECURITY_IN", input_filter,
         NULL, AP_FTYPE_CONTENT_SET);
+
+    /* Ensure that the output filter runs before other modules so that
+     * we get a request that has a better chance of not being modified:
+     *
+     * Currently:
+     *   mod_expires = -2
+     *   mod_cache   = -1
+     *   mod_deflate = -1
+     *   mod_headers =  0
+     */
     ap_register_output_filter("MODSECURITY_OUT", output_filter,
-        NULL, AP_FTYPE_CONTENT_SET);
+        NULL, AP_FTYPE_CONTENT_SET - 3);
 
     ap_register_output_filter("PDFP_OUT", pdfp_output_filter,
         NULL, AP_FTYPE_CONTENT_SET);
