@@ -26,17 +26,34 @@
 #include "msc_xml.h"
 
 /**
- * Log an alert message to the log, adding the rule metadata at the end.
+ * Format an alert message.
  */
-void msc_alert(modsec_rec *msr, int level, msre_actionset *actionset, const char *action_message,
+const char * msc_alert_message(modsec_rec *msr, msre_actionset *actionset, const char *action_message,
     const char *rule_message)
 {
     const char *message = NULL;
 
     if (rule_message == NULL) rule_message = "Unknown error.";
 
-    message = apr_psprintf(msr->mp, "%s %s%s", action_message,
-        rule_message, msre_format_metadata(msr, actionset));
+    if (action_message == NULL) {
+        message = apr_psprintf(msr->mp, "%s%s",
+            rule_message, msre_format_metadata(msr, actionset));
+    }
+    else {
+        message = apr_psprintf(msr->mp, "%s %s%s", action_message,
+            rule_message, msre_format_metadata(msr, actionset));
+    }
+
+    return message;
+}
+
+/**
+ * Log an alert message to the log, adding the rule metadata at the end.
+ */
+void msc_alert(modsec_rec *msr, int level, msre_actionset *actionset, const char *action_message,
+    const char *rule_message)
+{
+    const char *message = msc_alert_message(msr, actionset, action_message, rule_message);
 
     msr_log(msr, level, "%s", message);
 }
@@ -260,7 +277,7 @@ apr_status_t modsecurity_tx_init(modsec_rec *msr) {
     }
 
     /* Check if we are forcing buffering, then use memory only. */
-    if (msr->txcfg->reqbody_buffering) {
+    if (msr->txcfg->reqbody_buffering != REQUEST_BODY_FORCEBUF_OFF) {
         msr->msc_reqbody_storage = MSC_REQBODY_MEMORY;
         msr->msc_reqbody_spilltodisk = 0;
     }
