@@ -90,6 +90,9 @@ typedef struct msc_string msc_string;
 #define RESPONSE_BODY_LIMIT_ACTION_REJECT       0
 #define RESPONSE_BODY_LIMIT_ACTION_PARTIAL      1
 
+#define REQUEST_BODY_FORCEBUF_OFF               0
+#define REQUEST_BODY_FORCEBUF_ON                1
+
 #define SECACTION_TARGETS                       "REQUEST_URI"
 #define SECACTION_ARGS                          "@unconditionalMatch"
 
@@ -392,6 +395,10 @@ struct directory_config {
     /* AUDITLOG_SERIAL (single file) or AUDITLOG_CONCURRENT (multiple files) */
     int                  auditlog_type;
 
+    /* Mode for audit log directories and files */
+    apr_fileperms_t      auditlog_dirperms;
+    apr_fileperms_t      auditlog_fileperms;
+
     /* The name of the audit log file (for the old type), or the
      * name of the index file (for the new audit log type)
      */
@@ -425,7 +432,7 @@ struct directory_config {
     const char          *upload_dir;
     int                  upload_keep_files;
     int                  upload_validates_files;
-    int                  upload_filemode;
+    int                  upload_filemode; /* int only so NOT_SET works */
 
     /* Used only in the configuration phase. */
     msre_rule           *tmp_chain_starter;
@@ -545,6 +552,9 @@ apr_status_t DSOLOCAL modsecurity_request_body_retrieve(modsec_rec *msr, msc_dat
 void DSOLOCAL msc_add(modsec_rec *msr, int level, msre_actionset *actionset,
     const char *action_message, const char *rule_message);
 
+const char DSOLOCAL *msc_alert_message(modsec_rec *msr, msre_actionset *actionset, const char *action_message,
+    const char *rule_message);
+
 void DSOLOCAL msc_alert(modsec_rec *msr, int level, msre_actionset *actionset, const char *action_message,
     const char *rule_message);
 
