@@ -1,6 +1,6 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2008 Breach Security, Inc. (http://www.breach.com/)
+ * Copyright (c) 2004-2009 Breach Security, Inc. (http://www.breach.com/)
  *
  * This product is released under the terms of the General Public Licence,
  * version 2 (GPLv2). Please refer to the file LICENSE (included with this
@@ -309,6 +309,9 @@ apr_status_t modsecurity_request_body_store(modsec_rec *msr,
                 msr->msc_reqbody_processor);
             return -1;
         }
+    } else if (msr->txcfg->reqbody_buffering != REQUEST_BODY_FORCEBUF_OFF) {
+        /* Increase per-request data length counter if forcing buffering. */
+        msr->msc_reqbody_no_files_length += length;
     }
 
     /* Check that we are not over the request body no files limit. */
@@ -332,13 +335,12 @@ apr_status_t modsecurity_request_body_store(modsec_rec *msr,
 }
 
 /**
- *
+ * Replace a bunch of chunks holding a request body with a single large chunk.
  */
-static apr_status_t modsecurity_request_body_end_urlencoded(modsec_rec *msr, char **error_msg) {
+static apr_status_t modsecurity_request_body_end_raw(modsec_rec *msr, char **error_msg) {
     msc_data_chunk **chunks, *one_chunk;
     char *d;
     int i, sofar;
-    int invalid_count = 0;
 
     *error_msg = NULL;
 
@@ -349,12 +351,14 @@ static apr_status_t modsecurity_request_body_end_urlencoded(modsec_rec *msr, cha
             msr->msc_reqbody_length);
         return -1;
     }
+
     msr->msc_reqbody_buffer = malloc(msr->msc_reqbody_length + 1);
     if (msr->msc_reqbody_buffer == NULL) {
         *error_msg = apr_psprintf(msr->mp, "Unable to allocate memory to hold request body. Asked for %u bytes.",
             msr->msc_reqbody_length + 1);
         return -1;
     }
+
     msr->msc_reqbody_buffer[msr->msc_reqbody_length] = '\0';
 
     /* Copy the data we keep in chunks into the new buffer. */
@@ -388,12 +392,29 @@ static apr_status_t modsecurity_request_body_end_urlencoded(modsec_rec *msr, cha
         *error_msg = apr_pstrdup(msr->mp, "Failed to create structure to hold request body.");
         return -1;
     }
+
     one_chunk = (msc_data_chunk *)apr_pcalloc(msr->msc_reqbody_mp, sizeof(msc_data_chunk));
     one_chunk->data = msr->msc_reqbody_buffer;
     one_chunk->length = msr->msc_reqbody_length;
     one_chunk->is_permanent = 1;
     *(const msc_data_chunk **)apr_array_push(msr->msc_reqbody_chunks) = one_chunk;
 
+    return 1;
+}
+
+/**
+ *
+ */
+static apr_status_t modsecurity_request_body_end_urlencoded(modsec_rec *msr, char **error_msg) {
+    int invalid_count = 0;
+
+    *error_msg = NULL;
+
+    /* Create the raw buffer */
+    if (modsecurity_request_body_end_raw(msr, error_msg) != 1) {
+        return -1;
+    }
+
     /* Parse URL-encoded arguments in the request body. */
 
     if (parse_arguments(msr, msr->msc_reqbody_buffer, msr->msc_reqbody_length,
@@ -458,6 +479,9 @@ apr_status_t modsecurity_request_body_end(modsec_rec *msr, char **error_msg) {
                 return -1;
             }
         }
+    } else if (msr->txcfg->reqbody_buffering != REQUEST_BODY_FORCEBUF_OFF) {
+        /* Convert to a single continous buffer, but don't do anything else. */
+        return modsecurity_request_body_end_raw(msr, error_msg);
     }
 
     /* Note the request body no files length. */
