@@ -1,6 +1,6 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2008 Breach Security, Inc. (http://www.breach.com/)
+ * Copyright (c) 2004-2009 Breach Security, Inc. (http://www.breach.com/)
  *
  * This product is released under the terms of the General Public Licence,
  * version 2 (GPLv2). Please refer to the file LICENSE (included with this
@@ -16,6 +16,7 @@
  * directly using the email address support@breach.com.
  *
  */
+#include "msc_release.h"
 #include "msc_util.h"
 
 #include <ctype.h>
@@ -26,12 +27,28 @@
 
 #include <apr_lib.h>
 
-/* NOTE: Be careful as these can ONLY be used on static values for X.
+/**
+ * NOTE: Be careful as these can ONLY be used on static values for X.
  * (i.e. VALID_HEX(c++) will NOT work)
  */
 #define VALID_HEX(X) (((X >= '0')&&(X <= '9')) || ((X >= 'a')&&(X <= 'f')) || ((X >= 'A')&&(X <= 'F')))
 #define ISODIGIT(X) ((X >= '0')&&(X <= '7'))
 
+#if (defined(WIN32) || defined(NETWARE))
+/** Windows does not define all the octal modes */
+#define S_IXOTH 00001
+#define S_IWOTH 00002
+#define S_IROTH 00004
+#define S_IXGRP 00010
+#define S_IWGRP 00020
+#define S_IRGRP 00040
+#define S_IXUSR 00100
+#define S_IWUSR 00200
+#define S_IRUSR 00400
+#define S_ISVTX 01000
+#define S_ISGID 02000
+#define S_ISUID 04000
+#endif /* defined(WIN32 || NETWARE) */
 
 /**
  *
@@ -340,6 +357,17 @@ unsigned char x2c(unsigned char *what) {
     return digit;
 }
 
+/**
+ * Converts a single hexadecimal digit into a decimal value.
+ */
+unsigned char xsingle2c(unsigned char *what) {
+    register unsigned char digit;
+
+    digit = (what[0] >= 'A' ? ((what[0] & 0xdf) - 'A') + 10 : (what[0] - '0'));
+
+    return digit;
+}
+
 /**
  *
  */
@@ -406,17 +434,24 @@ char *current_filetime(apr_pool_t *mp) {
 /**
  *
  */
-int msc_mkstemp(char *template) {
+int msc_mkstemp_ex(char *template, int mode) {
     /* ENH Use apr_file_mktemp instead. */
 
     #if !(defined(WIN32)||defined(NETWARE))
     return mkstemp(template);
     #else
     if (mktemp(template) == NULL) return -1;
-    return open(template, O_WRONLY | O_APPEND | O_CREAT | O_BINARY, CREATEMODE_UNISTD);
+    return open(template, O_WRONLY | O_APPEND | O_CREAT | O_BINARY, mode);
     #endif
 }
 
+/**
+ *
+ */
+int msc_mkstemp(char *template) {
+    return msc_mkstemp_ex(template, CREATEMODE_UNISTD);
+}
+
 /**
  * Converts the input string to lowercase (in-place).
  */
@@ -650,7 +685,7 @@ int js_decode_nonstrict_inplace(unsigned char *input, long int input_len) {
                         j = 2;
                         buf[j] = '\0';
                     }
-                    *d++ = strtol(buf, NULL, 8);
+                    *d++ = (unsigned char)strtol(buf, NULL, 8);
                     i += 1 + j;
                     count++;
                 }
@@ -1158,21 +1193,11 @@ int normalise_path_inplace(unsigned char *input, int input_len, int win, int *ch
 }
 
 char *modsec_build(apr_pool_t *mp) {
-    int build_type = 0;
-    int i;
-
-    for (i = 0; modsec_build_type[i].name != NULL; i++) {
-        if (strcmp(MODSEC_VERSION_TYPE, modsec_build_type[i].name) == 0) {
-            build_type = modsec_build_type[i].val;
-            break;
-        }
-    }
-
     return apr_psprintf(mp, "%02i%02i%02i%1i%02i",
                             atoi(MODSEC_VERSION_MAJOR),
                             atoi(MODSEC_VERSION_MINOR),
                             atoi(MODSEC_VERSION_MAINT),
-                            build_type,
+                            get_modsec_build_type(NULL),
                             atoi(MODSEC_VERSION_RELEASE));
 }
 
@@ -1199,3 +1224,176 @@ char *resolve_relative_path(apr_pool_t *pool, const char *parent_filename, const
         strlen(parent_filename) - strlen(apr_filepath_name_get(parent_filename))),
         filename, NULL);
 }
+
+/**
+ * Decode a string that contains CSS-escaped characters.
+ * 
+ * References:
+ *     http://www.w3.org/TR/REC-CSS2/syndata.html#q4
+ *     http://www.unicode.org/roadmaps/
+ */
+int css_decode_inplace(unsigned char *input, long int input_len) {
+    unsigned char *d = (unsigned char *)input;
+    long int i, j, count;
+
+    if (input == NULL) return -1;
+
+    i = count = 0;
+    while (i < input_len) {
+
+        /* Is the character a backslash? */
+        if (input[i] == '\\') {
+
+            /* Is there at least one more byte? */
+            if (i + 1 < input_len) {
+                i++; /* We are not going to need the backslash. */
+
+                /* Check for 1-6 hex characters following the backslash */
+                j = 0;
+                while (    (j < 6)
+                        && (i + j < input_len)
+                        && (VALID_HEX(input[i + j])))
+                {
+                    j++;
+                }
+
+                if (j > 0) { /* We have at least one valid hexadecimal character. */
+                    int fullcheck = 0;
+
+                    /* For now just use the last two bytes. */
+                    switch (j) {
+                        /* Number of hex characters */
+                        case 1:
+                            *d++ = xsingle2c(&input[i]);
+                            break;
+
+                        case 2:
+                        case 3:
+                            /* Use the last two from the end. */
+                            *d++ = x2c(&input[i + j - 2]);
+                            break;
+
+                        case 4:
+                            /* Use the last two from the end, but request
+                             * a full width check.
+                             */
+                            *d = x2c(&input[i + j - 2]);
+                            fullcheck = 1;
+                            break;
+
+                        case 5:
+                            /* Use the last two from the end, but request
+                             * a full width check if the number is greater
+                             * or equal to 0xFFFF.
+                             */
+                            *d = x2c(&input[i + j - 2]);
+
+                            /* Do full check if first byte is 0 */
+                            if (input[i] == '0') {
+                                fullcheck = 1;
+                            }
+                            else {
+                                d++;
+                            }
+                            break;
+
+                        case 6:
+                            /* Use the last two from the end, but request
+                             * a full width check if the number is greater
+                             * or equal to 0xFFFF.
+                             */
+                            *d = x2c(&input[i + j - 2]);
+
+                            /* Do full check if first/second bytes are 0 */
+                            if (    (input[i] == '0')
+                                 && (input[i + 1] == '0')
+                            ) {
+                                fullcheck = 1;
+                            }
+                            else {
+                                d++;
+                            }
+                            break;
+                    }
+
+                    /* Full width ASCII (0xff01 - 0xff5e) needs 0x20 added */
+                    if (fullcheck) {
+                        if (   (*d > 0x00) && (*d < 0x5f)
+                            && ((input[i + j - 3] == 'f') ||
+                                (input[i + j - 3] == 'F'))
+                            && ((input[i + j - 4] == 'f') ||
+                                (input[i + j - 4] == 'F')))
+                        {
+                            (*d) += 0x20;
+                        }
+
+                        d++;
+                    }
+
+                    /* We must ignore a single whitespace after a hex escape */
+                    if ((i + j < input_len) && isspace(input[i + j])) {
+                        j++;
+                    }
+    
+                    /* Move over. */
+                    count++;
+                    i += j;
+                }
+
+                /* No hexadecimal digits after backslash */
+                else if (input[i] == '\n') {
+                    /* A newline character following backslash is ignored. */
+                    i++;
+                }
+
+                /* The character after backslash is not a hexadecimal digit, nor a newline. */
+                else {
+                    /* Use one character after backslash as is. */
+                    *d++ = input[i++];
+                    count++;
+                }
+            }
+            
+            /* No characters after backslash. */
+            else {
+                /* Do not include backslash in output (continuation to nothing) */
+                i++; 
+            }
+        }
+
+        /* Character is not a backslash. */
+        else {
+            /* Copy one normal character to output. */
+            *d++ = input[i++];
+            count++;
+        }
+    }
+
+    /* Terminate output string. */
+    *d = '\0';
+
+    return count;
+}
+
+/**
+ * Translate UNIX octal umask/mode to APR apr_fileperms_t
+ */
+apr_fileperms_t mode2fileperms(int mode) {
+    apr_fileperms_t perms = 0;
+
+    if (mode & S_IXOTH) perms |= APR_WEXECUTE;
+    if (mode & S_IWOTH) perms |= APR_WWRITE;
+    if (mode & S_IROTH) perms |= APR_WREAD;
+    if (mode & S_IXGRP) perms |= APR_GEXECUTE;
+    if (mode & S_IWGRP) perms |= APR_GWRITE;
+    if (mode & S_IRGRP) perms |= APR_GREAD;
+    if (mode & S_IXUSR) perms |= APR_UEXECUTE;
+    if (mode & S_IWUSR) perms |= APR_UWRITE;
+    if (mode & S_IRUSR) perms |= APR_UREAD;
+    if (mode & S_ISVTX) perms |= APR_WSTICKY;
+    if (mode & S_ISGID) perms |= APR_GSETID;
+    if (mode & S_ISUID) perms |= APR_USETID;
+
+    return perms;
+}
+
