@@ -68,9 +68,12 @@ char *msre_actionset_generate_action_string(apr_pool_t *pool, const msre_actions
     const apr_array_header_t *tarr = NULL;
     const apr_table_entry_t *telts = NULL;
     char *actions = NULL;
+    int chain;
     int i;
 
-    if (actionset == NULL) return apr_pstrdup(pool, "");
+    if (actionset == NULL) return NULL;
+
+    chain = ((actionset->rule != NOT_SET_P) && actionset->rule->chain_starter) ? 1 : 0;
 
     tarr = apr_table_elts(actionset->actions);
     telts = (const apr_table_entry_t*)tarr->elts;
@@ -79,6 +82,22 @@ char *msre_actionset_generate_action_string(apr_pool_t *pool, const msre_actions
         msre_action *action = (msre_action *)telts[i].val;
         int use_quotes = 0;
 
+        if (chain) {
+            /* Skip some actions that are not used in a chain. */
+            if (   (action->metadata->type == ACTION_DISRUPTIVE)
+                || (action->metadata->type == ACTION_METADATA)
+                || (strcmp("log", action->metadata->name) == 0)
+                || (strcmp("auditlog", action->metadata->name) == 0)
+                || (strcmp("nolog", action->metadata->name) == 0)
+                || (strcmp("noauditlog", action->metadata->name) == 0)
+                || (strcmp("severity", action->metadata->name) == 0)
+                || (strcmp("tag", action->metadata->name) == 0)
+                || (strcmp("phase", action->metadata->name) == 0)) 
+            {
+                continue;
+            }
+        }
+
         /* Check if we need any quotes */
         if (action->param != NULL) {
             int j;
@@ -102,7 +121,7 @@ char *msre_actionset_generate_action_string(apr_pool_t *pool, const msre_actions
             NULL);
     }
 
-    return (actions == NULL) ? apr_pstrdup(pool, "") : actions;
+    return actions;
 }
 
 /**
@@ -681,7 +700,7 @@ void msre_actionset_set_defaults(msre_actionset *actionset) {
     if (actionset->msg == NOT_SET_P) actionset->msg = NULL;
     if (actionset->logdata == NOT_SET_P) actionset->logdata = NULL;
     if (actionset->phase == NOT_SET) actionset->phase = 2;
-    if (actionset->severity == -1); /* leave at -1 */
+    if (actionset->severity == -1) {} /* leave at -1 */
     if (actionset->rule == NOT_SET_P) actionset->rule = NULL;
 
     /* Flow */
@@ -1169,8 +1188,9 @@ static msre_rule * msre_ruleset_fetch_phase_rule(const msre_ruleset *ruleset, co
     for (i = 0; i < phase_arr->nelts; i++) {
         msre_rule *rule = (msre_rule *)rules[i];
 
+        /* Rule with an action, not a sub-rule (chain) and a matching id */
         if (  (rule->actionset != NULL)
-           && !rule->actionset->is_chained
+           && (!rule->actionset->is_chained || !rule->chain_starter)
            && (rule->actionset->id != NULL)
            && (strcmp(rule->actionset->id, id) == 0))
         {
@@ -1641,8 +1661,22 @@ static void msre_perform_disruptive_actions(modsec_rec *msr, msre_rule *rule,
         || (msr->modsecurity->processing_mode == MODSEC_OFFLINE)
         || (actionset->intercept_action == ACTION_NONE))
     {
+        int log_level;
+
         /* If "nolog" was used log at a higher level to prevent an "alert". */
-        int log_level = (actionset->log == 0 ? 4 : 2);
+        if (actionset->log == 0) {
+            log_level = 4;
+
+            /* But, if "auditlog" is enabled, then still add the message. */
+            if (actionset->auditlog != 0) {
+                *(const char **)apr_array_push(msr->alerts) = msc_alert_message(msr, actionset, NULL, message);
+            }
+
+        }
+        else {
+            log_level = 2;
+        }
+
         msc_alert(msr, log_level, actionset, "Warning.", message);
 
         /* However, this will mark the txn relevant again if it is <= 3,
@@ -1743,9 +1777,16 @@ static int execute_operator(msre_var *var, msre_rule *rule, modsec_rec *msr,
     else {
         /* Match. */
         if (rc == 0) {
+            char *op_param = log_escape(msr->mp, rule->op_param);
+
+            /* Truncate op parameter. */
+            if (strlen(op_param) > 252) {
+                op_param = apr_psprintf(msr->mp, "%.252s ...", op_param);
+            }
+
             /* Operator did not match so we need to provide a message. */
             my_error_msg = apr_psprintf(msr->mp, "Match of \"%s %s\" against \"%s\" required.",
-                log_escape(msr->mp, rule->op_name), log_escape(msr->mp, rule->op_param),
+                log_escape(msr->mp, rule->op_name), op_param,
                 log_escape(msr->mp, full_varname));
         }
 
