@@ -1366,6 +1366,18 @@ static int var_multipart_missing_semicolon_generate(modsec_rec *msr, msre_var *v
     }
 }
 
+/* MULTIPART_INVALID_QUOTING */
+
+static int var_multipart_invalid_quoting_generate(modsec_rec *msr, msre_var *var, msre_rule *rule,
+    apr_table_t *vartab, apr_pool_t *mptmp)
+{
+    if ((msr->mpd != NULL)&&(msr->mpd->flag_invalid_quoting != 0)) {
+        return var_simple_generate(var, vartab, mptmp, "1");
+    } else {
+        return var_simple_generate(var, vartab, mptmp, "0");
+    }
+}
+
 /* MULTIPART_STRICT_ERROR */
 
 static int var_multipart_strict_error_generate(modsec_rec *msr, msre_var *var, msre_rule *rule,
@@ -1381,6 +1393,7 @@ static int var_multipart_strict_error_generate(modsec_rec *msr, msre_var *var, m
             ||(msr->mpd->flag_header_folding != 0)
             ||(msr->mpd->flag_lf_line != 0)
             ||(msr->mpd->flag_missing_semicolon != 0)
+            ||(msr->mpd->flag_invalid_quoting != 0)
         ) {
             return var_simple_generate(var, vartab, mptmp, "1");
         }
@@ -2282,7 +2295,7 @@ void msre_engine_register_default_variables(msre_engine *engine) {
     msre_engine_variable_register(engine,
         "GEO",
         VAR_LIST,
-        1, 1,
+        0, 1,
         var_generic_list_validate,
         var_geo_generate,
         VAR_DONT_CACHE, /* dynamic */
@@ -2454,6 +2467,17 @@ void msre_engine_register_default_variables(msre_engine *engine) {
         PHASE_REQUEST_BODY
     );
 
+    /* MULTIPART_INVALID_QUOTING */
+    msre_engine_variable_register(engine,
+        "MULTIPART_INVALID_QUOTING",
+        VAR_SIMPLE,
+        0, 0,
+        NULL,
+        var_multipart_invalid_quoting_generate,
+        VAR_DONT_CACHE, /* flag */
+        PHASE_REQUEST_BODY
+    );
+
     /* MULTIPART_STRICT_ERROR */
     msre_engine_variable_register(engine,
         "MULTIPART_STRICT_ERROR",
